package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ModelEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.CommonTreeModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-07-04 下午12:46
 * @Modify:
 **/
@Service
public class CoreMemTreeService implements ICoreMemTreeService {

    @Autowired
    private IPersonnelService personnelService;

    @Override
    public List<SysPosition> packTreeSysPosition(List<SysPosition> list, Long parentId) {
        List<SysPosition> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            List<SysPosition> copyList = new ArrayList<>(list);
            list.forEach(position -> {
                if(position.getParentId() != null && position.getParentId().equals(parentId)){
                    result.add(position);
                    copyList.remove(position);
                    position.setChildList(packTreeSysPosition(copyList, position.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysPosition> packTreeSysPositionBySysRefUserPosition(List<SysPosition> list, List<SysRefUserPosition> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref -> ids.add(ref.getPositionId()));
        }

        return packTreeSysPositionByIds(list, ids);
    }

    @Override
    public List<SysPosition> packTreeSysPositionByIds(List<SysPosition> list, List<Long> ids) {
        List<SysPosition> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            list.forEach(position -> {
                if(position.getChildList() == null || position.getChildList().size() <= 0){
                    if(ids.contains(position.getId())){
                        result.add(position);
                    }
                }else{
                    List<SysPosition> childList = packTreeSysPositionByIds(position.getChildList(), ids);
                    if(ids.contains(position.getId()) || childList != null && childList.size() > 0){
                        position.setChildList(childList);
                        result.add(position);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysPosition packTreeSysPositionById(List<SysPosition> list, Long id) {
        if(list != null && list.size() > 0){
            for (SysPosition position : list) {
                if(position.getId().equals(id)){
                    position.setChildList(packTreeSysPosition(list, position.getId()));
                    return position;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysPosition(SysPosition position) {
        List<Long> result = new ArrayList<>();

        if(position != null && position.getChildList() != null && position.getChildList().size() > 0){
            for (SysPosition sp : position.getChildList()) {
                result.add(sp.getId());
                result.addAll(findChildSysPosition(sp));
            }
        }

        return result;
    }

    @Override
    public List<Long> findSysPositionChildIdList(Long id) {
        SysPosition position = personnelService.getSysPositionById(id);
        if(position != null){
            List<SysPosition> list = personnelService.findSysPositionList();
            if(list != null && list.size() > 0){
                List<SysPosition> treeList = packTreeSysPosition(list, CommonEnumUtil.PARENT.DEFAULT.getValue());
                SysPosition treeSPosition = packTreeSysPositionById(treeList, position.getId());
                List<Long> idList = findChildSysPosition(treeSPosition);
                idList.add(position.getId());
                return idList;
            }
        }
        return null;
    }

    /********************************************* Start SysOrganize Tree *********************************************/
    @Override
    public List<SysOrganize> packTreeSysOrganize(List<SysOrganize> list, Long parentId) {
        List<SysOrganize> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            List<SysOrganize> copyList = new ArrayList<>(list);
            list.forEach(organize -> {
                if(organize.getParentId() != null && organize.getParentId().equals(parentId)){
                    result.add(organize);
                    copyList.remove(organize);
                    organize.setChildList(packTreeSysOrganize(copyList, organize.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeBySysRefUserOrganize(List<SysOrganize> list, List<SysRefUserDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref -> ids.add(ref.getOrganizeId()));
        }

        return packTreeSysOrganizeByIds(list, ids);
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeBySysRefUserInfoOrganize(List<SysOrganize> list, List<SysRefUserInfoDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref ->{
                ids.add(ref.getOrganizeId());
            });
        }

        return packTreeSysOrganizeByIds(list, ids);
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeByIds(List<SysOrganize> list, List<Long> ids) {
        List<SysOrganize> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            list.forEach(organize -> {
                if(organize.getChildList() == null || organize.getChildList().size() <= 0){
                    if(ids.contains(organize.getId())){
                        result.add(organize);
                    }
                }else{
                    List<SysOrganize> childList = packTreeSysOrganizeByIds(organize.getChildList(), ids);
                    if(ids.contains(organize.getId()) || childList != null && childList.size() > 0){
                        organize.setChildList(childList);
                        result.add(organize);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysOrganize packTreeSysOrganizeById(List<SysOrganize> list, Long id) {
        if(list != null && list.size() > 0){
            for (SysOrganize organize : list) {
                if(organize.getId().equals(id)){
                    organize.setChildList(packTreeSysOrganize(list, organize.getId()));
                    return organize;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysOrganize(SysOrganize organize) {
        List<Long> result = new ArrayList<>();

        if(organize != null && organize.getChildList() != null && organize.getChildList().size() > 0){
            for (SysOrganize so : organize.getChildList()) {
                result.add(so.getId());
                result.addAll(findChildSysOrganize(so));
            }
        }

        return result;
    }
    /********************************************* End SysOrganize Tree *********************************************/


    /********************************************* Start CommonTreeModel Tree *********************************************/

    @Override
    public List<CommonTreeModel> packTreeTreeModelUser(List<SysOrganize> organizeList, List<SysDepartment> departmentList, List<SysRefUserDepartment> refList) {
        if(refList != null && organizeList != null && departmentList != null){
            List<Long> organizeIdList = new ArrayList<>();
            List<Long> departmentIdList = new ArrayList<>();

            refList.forEach(ref -> {
                if(!organizeIdList.contains(ref.getOrganizeId())){
                    organizeIdList.add(ref.getOrganizeId());
                }
                if(!departmentIdList.contains(ref.getDepartmentId())){
                    departmentIdList.add(ref.getDepartmentId());
                }
            });

            organizeList = packTreeSysOrganize(organizeList, CommonEnumUtil.PARENT.DEFAULT.getValue());
            organizeList = packTreeSysOrganizeByIds(organizeList, organizeIdList);

            departmentList = packTreeSysDepartment(departmentList, CommonEnumUtil.PARENT.DEFAULT.getValue());
            departmentList = packTreeSysDepartmentByIds(departmentList, departmentIdList);

            List<CommonTreeModel> modelList = packTreeTreeModelSysOrganize(organizeList);
            List<CommonTreeModel> result = packTreeTreeModel(modelList, departmentList);
            return result;
        }

        return null;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModelSysOrganize(List<SysOrganize> list) {
        List<CommonTreeModel> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            for (SysOrganize organize : list) {
                CommonTreeModel model = new CommonTreeModel();
                model.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_ORGANIZE.getValue());
                model.setBean(organize);
                model.getChildList().addAll(packTreeTreeModelSysOrganize(organize.getChildList()));

                result.add(model);
            }
        }

        return result;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModelSysDepartment(List<SysDepartment> list) {
        List<CommonTreeModel> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            for (SysDepartment department : list) {
                CommonTreeModel model = new CommonTreeModel();
                model.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_DEPARTMENT.getValue());
                model.setBean(department);
                model.getChildList().addAll(packTreeTreeModelSysDepartment(department.getChildList()));

                result.add(model);
            }
        }

        return result;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModel(List<CommonTreeModel> list, List<SysDepartment> departmentList) {
        if(list != null && list.size() > 0 && departmentList != null && departmentList.size() > 0){
            list.forEach(model -> {
                SysOrganize organize = (SysOrganize) model.getBean();
                List<SysDepartment> departments = new ArrayList<>();
                departmentList.forEach(department -> {
                    if(department.getParentId().equals(CommonEnumUtil.PARENT.DEFAULT.getValue()) && department.getOrganizeId().equals(organize.getId())){
                        departments.add(department);
                    }
                });
                List<CommonTreeModel> childList = new ArrayList<>();
                childList.addAll(packTreeTreeModel(model.getChildList(), departmentList));
                childList.addAll(packTreeTreeModelSysDepartment(departments));
                model.setChildList(childList);
            });
        }
        return list;
    }

    /********************************************* Start CommonTreeModel Tree *********************************************/

    /********************************************* Start SysDepartment Tree *********************************************/
    @Override
    public List<SysDepartment> packTreeSysDepartment(List<SysDepartment> list, Long parentId) {
        List<SysDepartment> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            List<SysDepartment> copyList = new ArrayList<>(list);
            list.forEach(department -> {
                if(department.getParentId() != null && department.getParentId().equals(parentId)){
                    result.add(department);
                    copyList.remove(department);
                    department.setChildList(packTreeSysDepartment(copyList, department.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysDepartment> packTreeSysDepartmentBySysRefUserOrganize(List<SysDepartment> list, List<SysRefUserDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref -> ids.add(ref.getDepartmentId()));
        }

        return packTreeSysDepartmentByIds(list, ids);
    }

    @Override
    public List<SysDepartment> packTreeSysDepartmentBySysRefUserInfoOrganize(List<SysDepartment> list, List<SysRefUserInfoDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref -> ids.add(ref.getDepartmentId()));
        }

        return packTreeSysDepartmentByIds(list, ids);
    }

    @Override
    public List<SysDepartment> packTreeSysDepartmentByIds(List<SysDepartment> list, List<Long> ids) {
        List<SysDepartment> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            list.forEach(department -> {
                if(department.getChildList() == null || department.getChildList().size() <= 0){
                    if(ids.contains(department.getId())){
                        result.add(department);
                    }
                }else{
                    List<SysDepartment> childList = packTreeSysDepartmentByIds(department.getChildList(), ids);
                    if(ids.contains(department.getId()) || childList != null && childList.size() > 0){
                        department.setChildList(childList);
                        result.add(department);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysDepartment packTreeSysDepartmentById(List<SysDepartment> list, Long id) {
        if(list != null && list.size() > 0){
            for (SysDepartment department : list) {
                if(department.getId().equals(id)){
                    department.setChildList(packTreeSysDepartment(list, department.getId()));
                    return department;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysDepartment(SysDepartment department) {
        List<Long> result = new ArrayList<>();

        if(department != null && department.getChildList() != null && department.getChildList().size() > 0){
            for (SysDepartment dep : department.getChildList()) {
                result.add(dep.getId());
                result.addAll(findChildSysDepartment(dep));
            }
        }

        return result;
    }
    /********************************************* End SysDepartment Tree *********************************************/



}
